/* packet-wap.c
 * Routines for Wireless Application Protocol dissection
 * Copyright 2000, B.P. Fowler <BPFowler@leedsnet.uklinux.net>
 *
 * $Id: README.developer,v 1.7 2000/03/03 06:58:28 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@unicom.net>
 * Copyright 1998 Gerald Combs
 *
 * Copied from WHATEVER_FILE_YOU_USED
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif

#ifdef NEED_SNPRINTF_H
# ifdef HAVE_STDARG_H
#  include <stdarg.h>
# else
#  include <varargs.h>
# endif
# include "snprintf.h"
#endif

#include <string.h>
#include <glib.h>
#include "packet.h"
#include "packet-wap.h"

/* File scoped variables for the protocol and registered fields */
static int proto_wap = HF_EMPTY;
static int hf_wap_header_fixed_part = HF_EMPTY;
static int hf_wap_header_variable_part = HF_EMPTY;
static int hf_wap_data = HF_EMPTY;

/* Initialize the subtree pointers */
static gint ett_wap = ETT_EMPTY;

/* Code to actually dissect the packets */
void
dissect_wap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
{

/* Set up structures we will need to add the protocol subtree and manage it */
	proto_item *ti;
	proto_tree *wap_tree;
	
/* Make entries in Protocol column and Info column on summary display */
	if (check_col(fd, COL_PROTOCOL)) 
		col_add_str(fd, COL_PROTOCOL, "WAP");
    
/* This field shows up as the "Info" column in the display; you should make
   it, if possible, summarize what's in the packet, so that a user looking
   at the list of packets can tell what type of packet it is.

   "col_add_fstr()" can be used instead of "col_add_str()"; it takes
   "printf()"-like arguments. */
	if (check_col(fd, COL_INFO)) 
		col_add_str(fd, COL_INFO, "WAP");

/* In the interest of speed, if "tree" is NULL, don't do any work not
   necessary to generate protocol tree items. */
	if (tree) {
/* NOTE: The offset and length values in the previous call to
   "proto_tree_add_item()" define what data bytes to highlight in the hex
   display window when the line in the protocol tree display
   corresponding to that item is selected.

   END_OF_FRAME is a handy way to highlight all data from the offset to
   the end of the packet. */
		ti = proto_tree_add_item(tree, proto_wap, offset, 1, NULL);
	        wap_tree = proto_item_add_subtree(ti, ett_wap);

/* Code to process the packet goes here */
		{
			unsigned char  b0 = pd[offset + 0];
			unsigned char  b1 = pd[offset + 1];
			unsigned char  b2 = pd[offset + 2];
			unsigned char  b3 = pd[offset + 3];
			unsigned char  fCon = b0 & 0x80;

			ti = proto_tree_add_item_format(
					wap_tree, 			/* tree */
					hf_wap_header_fixed_part, 	/* id */
					offset, 			/* start */
					4,				/* length */
					"What should go here!",		/* value */
					"Header (Fixed part) %02X %02X %02X %02X"  ,			/* format */
					b0, b1, b2, b3
			     );
			if( fCon ) {
				/* There is a variable part if the Con flag is set */ 
				ti = proto_tree_add_item_format(
						wap_tree, 			/* tree */
						hf_wap_header_variable_part, 	/* id */
						offset + 4, 			/* start */
						4,				/* length */
						"What should go here!",		/* value */
						"Header (Variable part) %02X %02X %02X %02X"  ,			/* format */
						0, 1, 2, 3
				    );
			}; /* End of variable part of header */
			ti = proto_tree_add_item_format(
					wap_tree, 			/* tree */
					hf_wap_data, 	/* id */
					offset, 			/* start */
					END_OF_FRAME,				/* length */
					"What should go here!",		/* value */
					"Payload"  			/* format */
			     );
		}
	}
}

/* Register the protocol with Ethereal */
void
proto_register_wap(void)
{                 

/* Setup list of header fields */
	static hf_register_info hf[] = {
		{ &hf_wap_header_fixed_part,
			{ 	"Header: Fixed part",           
				"wap.header_fixed_part",
				FT_BYTES, BASE_HEX, NULL, 0x0,          
				"Fixed part of the header" 
			}
		},
		{ &hf_wap_header_variable_part,
			{ 	"Header: Variable part",           
				"wap.header_variable_part",
				FT_BYTES, BASE_HEX, NULL, 0x0,          
				"Variable part of the header" 
			}
		},
		{ &hf_wap_data,
			{ 	"Data",           
				"wap.header_data",
				FT_BYTES, BASE_HEX, NULL, 0x0,          
				"Data" 
			}
		},
	};
	
/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_wap,
	};

/* Register the protocol name and description */
	proto_wap = proto_register_protocol("Wireless Application Protocol", "wap");

/* Required function calls to register the header fields and subtrees used */
	proto_register_field_array(proto_wap, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
};
